from django.db import models
from django.contrib.auth.models import AbstractUser, Group
from xlibrary.fields import ForeignKeyOptional
# from accountinginfo.models import Ledger
from invitations.base_invitation import AbstractBaseInvitation
from django.utils.translation import ugettext_lazy as _
from invitations.app_settings import app_settings
from django.utils.crypto import get_random_string
from django.utils import timezone
import datetime
from django.contrib.sites.models import Site
from django.dispatch import receiver
from invitations.signals import invite_accepted
from invitations.adapters import get_invitations_adapter
from invitations import signals
from django.conf import settings
from django.core.exceptions import PermissionDenied
# from invitations.utils import get_invitation_model
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse
# Create your models here.


class xManager(models.Manager):
    def __init__(self, key=''):
        super(xManager, self).__init__()
        self.k = key

    def for_user(self, user, methodname=''):
        if methodname == 'GET' and user.has_perm(self.model._meta.app_label + '.view_' + self.model.__name__.lower()) == False:
            # return super(xManager, self).get_queryset().none()
            raise PermissionDenied()
        if user.is_superuser:
            return super(xManager, self).get_queryset()
        elif user.is_staff:
            if user.groups.first().option.privilege == 'GROUP':
                return super(xManager, self).get_queryset().filter(user_created__groups__in=user.groups.all())
            else:
                return super(xManager, self).get_queryset()
        else:
            if self.k != '':
                return super(xManager, self).get_queryset().filter(**{self.k: user})
            else:
                return super(xManager, self).get_queryset().none()

    def all(self):
        return super(xManager, self).get_queryset()


class GroupOptions(models.Model):
    PRIVILEGE_CHOICES = (
        ('ALL', 'Can access all permitted data'),
        ('GROUP', 'Can access permitted data belonging to group only'),
        ('SELF', 'Can access permitted data belonging to the logged in user only'),
    )
    groups = models.OneToOneField(
        Group, on_delete=models.CASCADE, primary_key=True, related_name='FK_%(class)s_option')
    privilege = models.CharField(
        max_length=20, choices=PRIVILEGE_CHOICES, null=False, default='ALL',)

    def delete(self, *args, **kwargs):
        self.groups.delete()
        return super(self.__class__, self).delete(*args, **kwargs)


class User(AbstractUser):
    is_admin=models.BooleanField(default=False, help_text='Designates whether the user is an admin.', verbose_name='admin status')
    class Meta:
        verbose_name = "User"
        verbose_name_plural = "Users"


class GuestInvitation(AbstractBaseInvitation):
    email = models.EmailField(unique=True, verbose_name=_('e-mail address'),
                              max_length=app_settings.EMAIL_MAX_LENGTH)
    created = models.DateTimeField(verbose_name=_('created'),
                                   default=timezone.now)
    authorentity = models.IntegerField('author',null=True,blank=True)

    @classmethod
    def create(cls, email, inviter=None, authorentity=None,  **kwargs):
        key = get_random_string(64).lower()
        instance = cls._default_manager.create(
            email=email,
            key=key,
            inviter=inviter,
            authorentity=authorentity,
            **kwargs)
        return instance

    def key_expired(self):
        expiration_date = (
            self.sent + datetime.timedelta(
                days=app_settings.INVITATION_EXPIRY))
        return expiration_date <= timezone.now()

    def send_invitation(self, request, **kwargs):
        current_site = kwargs.pop('site', Site.objects.get_current())
        invite_url = reverse('invitations:accept-invite',
                             args=[self.key])
        invite_url = request.build_absolute_uri(invite_url)
        ctx = kwargs
        ctx.update({
            'invite_url': invite_url,
            'site_name': current_site.name,
            'email': self.email,
            'key': self.key,
            'inviter': self.inviter,
        })
        email_template = 'invitations/email/email_invite'

        get_invitations_adapter().send_mail(
            email_template,
            self.email,
            ctx)
        self.sent = timezone.now()
        self.save()

        signals.invite_url_sent.send(
            sender=self.__class__,
            instance=self,
            invite_url_sent=invite_url,
            inviter=self.inviter)

    class Meta:
        verbose_name = "Guest Invitation"
        verbose_name_plural = "Guest Invitations"


# @receiver(invite_accepted)
# def update_ledger(sender, email, **kwargs):
#     try:
#         # Invitation = get_invitation_model()  # Get the Invitation model
#         # Grab the Invitation instance
#         # invite = Invitation.objects.get(email=email)
#         invite=GuestInvitation.objects.get(email=email)
#         user = User.objects.create(email=email)
#         invite.authorentity.guestlogin = user  # Pass your invitation's patient to the related user
#         invite.authorentity.save()
#     except GuestInvitation.DoesNotExist:
#         print("this was probably not an invited user.")



from django.apps import apps as django_apps
from django.core.exceptions import ImproperlyConfigured

from django.conf import settings


def get_author_model():
    """
    Returns the Invitation model that is active in this project.
    """
    path = getattr(settings, 'INVITATIONS_AUTHOR_MODEL')
    try:
        return django_apps.get_model(path)
        print(django_apps.get_model(path).Meta.verbose_name)
    except ValueError:
        raise ImproperlyConfigured(
            "path must be of the form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "path refers to model that\
             has not been installed"
        )



# Generated by Django 2.2.17 on 2021-05-16 08:22

from django.conf import settings
import django.contrib.auth.models
import django.contrib.auth.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone

class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
	('is_admin',models.BooleanField(default=False, help_text='Designates whether the user is an admin.', verbose_name='admin status')),
            ],
            options={
                'verbose_name': 'User',
                'verbose_name_plural': 'Users',
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='GroupOptions',
            fields=[
                ('groups', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='option', serialize=False, to='auth.Group')),
                ('privilege', models.CharField(choices=[('ALL', 'Can access all permitted data'), ('GROUP', 'Can access permitted data belonging to group only'), ('SELF', 'Can access permitted data belonging to the logged in user only')], default='ALL', max_length=20)),
            ],
        ),
        migrations.CreateModel(
            name='GuestInvitation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('accepted', models.BooleanField(default=False, verbose_name='accepted')),
                ('key', models.CharField(max_length=64, unique=True, verbose_name='key')),
                ('sent', models.DateTimeField(null=True, verbose_name='sent')),
                ('email', models.EmailField(max_length=254, unique=True, verbose_name='e-mail address')),
                ('created', models.DateTimeField(default=django.utils.timezone.now, verbose_name='created')),
                ('authorentity', models.IntegerField(blank=True, null=True, verbose_name='author')),
                ('inviter', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Guest Invitation',
                'verbose_name_plural': 'Guest Invitations',
            },
        ),
    ]
